package org.picketlink.federation.examples.auth;

import java.io.InputStream;
import java.security.Principal;

import javax.security.jacc.PolicyContext;
import javax.security.jacc.PolicyContextException;
import javax.servlet.http.HttpServletRequest;

import org.picketlink.identity.federation.api.saml.v2.response.SAML2Response;
import org.picketlink.identity.federation.bindings.jboss.auth.SAML2LoginModule;
import org.picketlink.identity.federation.bindings.tomcat.sp.holder.ServiceProviderSAMLContext;
import org.picketlink.identity.federation.core.exceptions.ConfigurationException;
import org.picketlink.identity.federation.core.exceptions.ParsingException;
import org.picketlink.identity.federation.core.exceptions.ProcessingException;
import org.picketlink.identity.federation.saml.v2.assertion.AssertionType;
import org.picketlink.identity.federation.saml.v2.protocol.ResponseType;
import org.picketlink.identity.federation.web.util.PostBindingUtil;
import org.picketlink.identity.federation.web.util.RedirectBindingUtil;
import org.w3c.dom.Element;

public class CustomSAML2LoginModule extends SAML2LoginModule {

    private static final String HTTP_REQUEST_POLICY_CONTEXT = "javax.servlet.http.HttpServletRequest";
    private static final String SAML_RESPONSE_REQUEST_PARAM_NAME = "SAMLResponse";

    /*
     * (non-Javadoc)
     * 
     * @see org.picketlink.identity.federation.bindings.jboss.auth.SAML2LoginModule #getIdentity()
     */
    protected Principal getIdentity() {
        SAMLPrincipal simplePrincipal = null;

        try {
            // creates a custom Principal with the issued SAML Assertion returned from the IDP.
            simplePrincipal = new SAMLPrincipal(ServiceProviderSAMLContext.getUserName(),
                    getAssertionElement(getSAMLResponseInputStream()));
        } catch (Exception e) {
            e.printStackTrace();
        }

        return simplePrincipal;
    }

    /**
     * <p>
     * Returns the SAML Assertion issued from the IDP.
     * </p>
     * 
     * @param samlResponseInputStream
     * @return
     * @throws ParsingException
     * @throws ConfigurationException
     * @throws ProcessingException
     * @throws Exception
     */
    private Element getAssertionElement(InputStream samlResponseInputStream) throws ParsingException, ConfigurationException,
            ProcessingException, Exception {
        ResponseType responseType = (ResponseType) new SAML2Response().getSAML2ObjectFromStream(samlResponseInputStream);
        AssertionType assertionType = responseType.getAssertions().get(0).getAssertion();

        return org.picketlink.identity.federation.core.wstrust.plugins.saml.SAMLUtil.toElement(assertionType);
    }

    /**
     * <p>
     * Returns the SAML Assertion from the request and creates a {@link InputStream}.
     * </p>
     * 
     * @return
     * @throws PolicyContextException
     */
    private InputStream getSAMLResponseInputStream() throws PolicyContextException {
        String samlResponseParam = getCurrentHttpRequest().getParameter(SAML_RESPONSE_REQUEST_PARAM_NAME);
        InputStream samlResponseInputStream = null;

        if (getCurrentHttpRequest().getMethod().equals("POST")) {
            samlResponseInputStream = PostBindingUtil.base64DecodeAsStream(samlResponseParam);
        } else {
            samlResponseInputStream = RedirectBindingUtil.base64DeflateDecode(samlResponseParam);
        }

        return samlResponseInputStream;
    }

    /**
     * <p>
     * Returns the current {@ HttpServletRequest} instance. 
     * </p>
     * 
     * @return
     * @throws PolicyContextException
     */
    private HttpServletRequest getCurrentHttpRequest() throws PolicyContextException {
        return (HttpServletRequest) PolicyContext.getContext(HTTP_REQUEST_POLICY_CONTEXT);
    }

}
